@@ -37,6 +37,7 @@ import javax.jms.JMSException;
 import javax.jms.JMSSecurityRuntimeException;
 import javax.net.ssl.SSLContext;
 
+import org.apache.qpid.jms.JmsConnectionExtensions;
 import org.apache.qpid.jms.JmsTemporaryDestination;
 import org.apache.qpid.jms.message.JmsInboundMessageDispatch;
 import org.apache.qpid.jms.message.JmsMessageFactory;
@@ -57,8 +58,11 @@ import org.apache.qpid.jms.provider.NoOpAsyncResult;
 import org.apache.qpid.jms.provider.Provider;
 import org.apache.qpid.jms.provider.ProviderClosedException;
 import org.apache.qpid.jms.provider.ProviderConstants.ACK_TYPE;
+import org.apache.qpid.jms.provider.ProviderFailedException;
 import org.apache.qpid.jms.provider.ProviderFuture;
+import org.apache.qpid.jms.provider.ProviderFutureFactory;
 import org.apache.qpid.jms.provider.ProviderListener;
+import org.apache.qpid.jms.provider.ProviderSynchronization;
 import org.apache.qpid.jms.provider.amqp.builders.AmqpClosedConnectionBuilder;
 import org.apache.qpid.jms.provider.amqp.builders.AmqpConnectionBuilder;
 import org.apache.qpid.jms.sasl.Mechanism;
@@ -80,13 +84,13 @@ import org.apache.qpid.proton.engine.SaslListener;
 import org.apache.qpid.proton.engine.impl.CollectorImpl;
 import org.apache.qpid.proton.engine.impl.ProtocolTracer;
 import org.apache.qpid.proton.engine.impl.TransportImpl;
+import org.apache.qpid.proton.engine.impl.TransportInternal;
 import org.apache.qpid.proton.framing.TransportFrame;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
 import io.netty.buffer.ByteBuf;
 import io.netty.buffer.ByteBufUtil;
-import io.netty.util.ReferenceCountUtil;
 
 /**
  * An AMQP v1.0 Provider.
@@ -100,7 +104,7 @@ import io.netty.util.ReferenceCountUtil;
  * All work within this Provider is serialized to a single Thread.  Any asynchronous exceptions
  * will be dispatched from that Thread and all in-bound requests are handled there as well.
  */
-public class AmqpProvider implements Provider, TransportListener , AmqpResourceParent, SaslListener {
+public class AmqpProvider implements Provider, TransportListener , AmqpResourceParent {
 
     private static final Logger LOG = LoggerFactory.getLogger(AmqpProvider.class);
 
@@ -113,6 +117,8 @@ public class AmqpProvider implements Provider, TransportListener , AmqpResourceP
     private static final AtomicInteger PROVIDER_SEQUENCE = new AtomicInteger();
     private static final NoOpAsyncResult NOOP_REQUEST = new NoOpAsyncResult();
 
+    private static final int DEFAULT_MAX_WRITE_BYTES_BEFORE_FLUSH = 128 * 1024;
+
     private volatile ProviderListener listener;
     private volatile AmqpConnection connection;
     private AmqpSaslAuthenticator authenticator;
@@ -128,17 +134,20 @@ public class AmqpProvider implements Provider, TransportListener , AmqpResourceP
     private int drainTimeout = 60000;
     private long sessionOutoingWindow = -1; // Use proton default
     private int maxFrameSize = DEFAULT_MAX_FRAME_SIZE;
+    private int maxWriteBytesBeforeFlush = DEFAULT_MAX_WRITE_BYTES_BEFORE_FLUSH;
 
     private boolean allowNonSecureRedirects;
 
     private final URI remoteURI;
     private final AtomicBoolean closed = new AtomicBoolean();
+    private volatile Throwable failureCause;
     private ScheduledThreadPoolExecutor serializer;
     private final org.apache.qpid.proton.engine.Transport protonTransport =
         org.apache.qpid.proton.engine.Transport.Factory.create();
     private final Collector protonCollector = new CollectorImpl();
     private final Connection protonConnection = Connection.Factory.create();
 
+    private final ProviderFutureFactory futureFactory;
     private AsyncResult connectionRequest;
     private ScheduledFuture<?> nextIdleTimeoutCheck;
 
@@ -149,10 +158,13 @@ public class AmqpProvider implements Provider, TransportListener , AmqpResourceP
      *        The URI of the AMQP broker this Provider instance will connect to.
      * @param transport
      * 		  The underlying Transport that will be used for wire level communications.
+     * @param futureFactory
+     * 		  The ProviderFutureFactory to use when futures are requested.
      */
-    public AmqpProvider(URI remoteURI, Transport transport) {
+    public AmqpProvider(URI remoteURI, Transport transport, ProviderFutureFactory futureFactory) {
         this.remoteURI = remoteURI;
         this.transport = transport;
+        this.futureFactory = futureFactory;
 
         serializer = new ScheduledThreadPoolExecutor(1, new QpidJMSThreadFactory(
             "AmqpProvider :(" + PROVIDER_SEQUENCE.incrementAndGet() + "):[" +
@@ -164,9 +176,9 @@ public class AmqpProvider implements Provider, TransportListener , AmqpResourceP
 
     @Override
     public void connect(final JmsConnectionInfo connectionInfo) throws IOException {
-        checkClosed();
+        checkClosedOrFailed();
 
-        final ProviderFuture connectRequest = new ProviderFuture();
+        final ProviderFuture connectRequest = futureFactory.createFuture();
 
         serializer.execute(new Runnable() {
 
@@ -179,8 +191,16 @@ public class AmqpProvider implements Provider, TransportListener , AmqpResourceP
                 try {
                     protonTransport.setEmitFlowEventOnSend(false);
 
+                    try {
+                        ((TransportInternal) protonTransport).setUseReadOnlyOutputBuffer(false);
+                    } catch (NoSuchMethodError nsme) {
+                        // using a version at runtime where the optimisation isn't available, ignore
+                        LOG.trace("Proton output buffer optimisation unavailable");
+                    }
+
                     if (getMaxFrameSize() > 0) {
                         protonTransport.setMaxFrameSize(getMaxFrameSize());
+                        protonTransport.setOutboundFrameSizeLimit(getMaxFrameSize());
                     }
 
                     protonTransport.setChannelMax(getChannelMax());
@@ -188,10 +208,28 @@ public class AmqpProvider implements Provider, TransportListener , AmqpResourceP
                     protonTransport.bind(protonConnection);
                     protonConnection.collect(protonCollector);
 
-                    SSLContext sslContextOverride = connectionInfo.getSslContextOverride();
+                    final SSLContext sslContextOverride;
+                    if (connectionInfo.getExtensionMap().containsKey(JmsConnectionExtensions.SSL_CONTEXT)) {
+                        sslContextOverride =
+                            (SSLContext) connectionInfo.getExtensionMap().get(
+                                JmsConnectionExtensions.SSL_CONTEXT).apply(connectionInfo.getConnection(), transport.getRemoteLocation());
+                    } else {
+                        sslContextOverride = null;
+                    }
 
                     transport.setTransportListener(AmqpProvider.this);
                     transport.setMaxFrameSize(maxFrameSize);
+
+                    if (connectionInfo.getExtensionMap().containsKey(JmsConnectionExtensions.HTTP_HEADERS_OVERRIDE)) {
+                        @SuppressWarnings({ "unchecked" })
+                        Map<String, String> headers = (Map<String, String>)
+                            connectionInfo.getExtensionMap().get(
+                                JmsConnectionExtensions.HTTP_HEADERS_OVERRIDE).apply(connectionInfo.getConnection(), transport.getRemoteLocation());
+                        if (headers != null) {
+                            transport.getTransportOptions().getHttpHeaders().putAll(headers);
+                        }
+                    }
+
                     transport.connect(sslContextOverride);
 
                     if (saslLayer) {
@@ -206,7 +244,36 @@ public class AmqpProvider implements Provider, TransportListener , AmqpResourceP
                         }
 
                         sasl.setRemoteHostname(hostname);
-                        sasl.setListener(AmqpProvider.this);
+                        sasl.setListener(new SaslListener() {
+
+                            @Override
+                            public void onSaslMechanisms(Sasl sasl, org.apache.qpid.proton.engine.Transport transport) {
+                                authenticator.handleSaslMechanisms(sasl, transport);
+                                checkSaslAuthenticationState();
+                            }
+
+                            @Override
+                            public void onSaslChallenge(Sasl sasl, org.apache.qpid.proton.engine.Transport transport) {
+                                authenticator.handleSaslChallenge(sasl, transport);
+                                checkSaslAuthenticationState();
+                            }
+
+                            @Override
+                            public void onSaslOutcome(Sasl sasl, org.apache.qpid.proton.engine.Transport transport) {
+                                authenticator.handleSaslOutcome(sasl, transport);
+                                checkSaslAuthenticationState();
+                            }
+
+                            @Override
+                            public void onSaslInit(Sasl sasl, org.apache.qpid.proton.engine.Transport transport) {
+                                // Server only event
+                            }
+
+                            @Override
+                            public void onSaslResponse(Sasl sasl, org.apache.qpid.proton.engine.Transport transport) {
+                                // Server only event
+                            }
+                        });
 
                         authenticator = new AmqpSaslAuthenticator((remoteMechanisms) -> findSaslMechanism(remoteMechanisms));
 
@@ -231,7 +298,7 @@ public class AmqpProvider implements Provider, TransportListener , AmqpResourceP
 
     @Override
     public void start() throws IOException, IllegalStateException {
-        checkClosed();
+        checkClosedOrFailed();
 
         if (listener == null) {
             throw new IllegalStateException("No ProviderListener registered.");
@@ -241,15 +308,7 @@ public class AmqpProvider implements Provider, TransportListener , AmqpResourceP
     @Override
     public void close() {
         if (closed.compareAndSet(false, true)) {
-            final ProviderFuture request = new ProviderFuture() {
-
-                @Override
-                public void onFailure(Throwable result) {
-                    // During close it is fine if the close call fails
-                    // this in unrecoverable so we just log the event.
-                    onSuccess();
-                }
-            };
+            final ProviderFuture request = futureFactory.createUnfailableFuture();
 
             serializer.execute(new Runnable() {
 
@@ -324,13 +383,13 @@ public class AmqpProvider implements Provider, TransportListener , AmqpResourceP
 
     @Override
     public void create(final JmsResource resource, final AsyncResult request) throws IOException, JMSException {
-        checkClosed();
+        checkClosedOrFailed();
         serializer.execute(new Runnable() {
 
             @Override
             public void run() {
                 try {
-                    checkClosed();
+                    checkClosedOrFailed();
                     resource.visit(new JmsResourceVistor() {
                         @Override
                         public void processSessionInfo(JmsSessionInfo sessionInfo) throws Exception {
@@ -345,7 +404,14 @@ public class AmqpProvider implements Provider, TransportListener , AmqpResourceP
 
                         @Override
                         public void processConsumerInfo(JmsConsumerInfo consumerInfo) throws Exception {
-                            AmqpSession session = connection.getSession(consumerInfo.getParentId());
+                            final AmqpSession session;
+
+                            if (consumerInfo.isConnectionConsumer()) {
+                                session = connection.getConnectionSession();
+                            } else {
+                                session = connection.getSession(consumerInfo.getParentId());
+                            }
+
                             session.createConsumer(consumerInfo, request);
                         }
 
@@ -401,13 +467,13 @@ public class AmqpProvider implements Provider, TransportListener , AmqpResourceP
 
     @Override
     public void start(final JmsResource resource, final AsyncResult request) throws IOException {
-        checkClosed();
+        checkClosedOrFailed();
         serializer.execute(new Runnable() {
 
             @Override
             public void run() {
                 try {
-                    checkClosed();
+                    checkClosedOrFailed();
                     resource.visit(new JmsDefaultResourceVisitor() {
 
                         @Override
@@ -428,13 +494,13 @@ public class AmqpProvider implements Provider, TransportListener , AmqpResourceP
 
     @Override
     public void stop(final JmsResource resource, final AsyncResult request) throws IOException {
-        checkClosed();
+        checkClosedOrFailed();
         serializer.execute(new Runnable() {
 
             @Override
             public void run() {
                 try {
-                    checkClosed();
+                    checkClosedOrFailed();
                     resource.visit(new JmsDefaultResourceVisitor() {
 
                         @Override
@@ -455,13 +521,13 @@ public class AmqpProvider implements Provider, TransportListener , AmqpResourceP
 
     @Override
     public void destroy(final JmsResource resource, final AsyncResult request) throws IOException {
-        checkClosed();
+        checkClosedOrFailed();
         serializer.execute(new Runnable() {
 
             @Override
             public void run() {
                 try {
-                    checkClosed();
+                    checkClosedOrFailed();
                     resource.visit(new JmsDefaultResourceVisitor() {
 
                         @Override
@@ -557,24 +623,15 @@ public class AmqpProvider implements Provider, TransportListener , AmqpResourceP
 
     @Override
     public void send(final JmsOutboundMessageDispatch envelope, final AsyncResult request) throws IOException {
-        checkClosed();
+        checkClosedOrFailed();
         serializer.execute(new Runnable() {
 
             @Override
             public void run() {
                 try {
-                    checkClosed();
-
+                    checkClosedOrFailed();
                     JmsProducerId producerId = envelope.getProducerId();
-                    AmqpProducer producer = null;
-
-                    if (producerId.getProviderHint() instanceof AmqpFixedProducer) {
-                        producer = (AmqpFixedProducer) producerId.getProviderHint();
-                    } else {
-                        AmqpSession session = connection.getSession(producerId.getParentId());
-                        producer = session.getProducer(producerId);
-                    }
-
+                    AmqpProducer producer = (AmqpProducer) producerId.getProviderHint();
                     producer.send(envelope, request);
                 } catch (Throwable t) {
                     request.onFailure(t);
@@ -585,13 +642,13 @@ public class AmqpProvider implements Provider, TransportListener , AmqpResourceP
 
     @Override
     public void acknowledge(final JmsSessionId sessionId, final ACK_TYPE ackType, final AsyncResult request) throws IOException {
-        checkClosed();
+        checkClosedOrFailed();
         serializer.execute(new Runnable() {
 
             @Override
             public void run() {
                 try {
-                    checkClosed();
+                    checkClosedOrFailed();
                     AmqpSession amqpSession = connection.getSession(sessionId);
                     amqpSession.acknowledge(ackType);
                     pumpToProtonTransport(request);
@@ -605,23 +662,16 @@ public class AmqpProvider implements Provider, TransportListener , AmqpResourceP
 
     @Override
     public void acknowledge(final JmsInboundMessageDispatch envelope, final ACK_TYPE ackType, final AsyncResult request) throws IOException {
-        checkClosed();
+        checkClosedOrFailed();
         serializer.execute(new Runnable() {
 
             @Override
             public void run() {
                 try {
-                    checkClosed();
+                    checkClosedOrFailed();
 
                     JmsConsumerId consumerId = envelope.getConsumerId();
-                    AmqpConsumer consumer = null;
-
-                    if (consumerId.getProviderHint() instanceof AmqpConsumer) {
-                        consumer = (AmqpConsumer) consumerId.getProviderHint();
-                    } else {
-                        AmqpSession session = connection.getSession(consumerId.getParentId());
-                        consumer = session.getConsumer(consumerId);
-                    }
+                    AmqpConsumer consumer = (AmqpConsumer) consumerId.getProviderHint();
 
                     consumer.acknowledge(envelope, ackType);
 
@@ -629,8 +679,9 @@ public class AmqpProvider implements Provider, TransportListener , AmqpResourceP
                         request.onSuccess();
                         pumpToProtonTransport(request);
                     } else {
-                        pumpToProtonTransport(request);
+                        pumpToProtonTransport(request, false);
                         request.onSuccess();
+                        transport.flush();
                     }
                 } catch (Throwable t) {
                     request.onFailure(t);
@@ -641,13 +692,13 @@ public class AmqpProvider implements Provider, TransportListener , AmqpResourceP
 
     @Override
     public void commit(final JmsTransactionInfo transactionInfo, JmsTransactionInfo nextTransactionId, final AsyncResult request) throws IOException {
-        checkClosed();
+        checkClosedOrFailed();
         serializer.execute(new Runnable() {
 
             @Override
             public void run() {
                 try {
-                    checkClosed();
+                    checkClosedOrFailed();
                     AmqpSession session = connection.getSession(transactionInfo.getSessionId());
                     session.commit(transactionInfo, nextTransactionId, request);
                     pumpToProtonTransport(request);
@@ -660,13 +711,13 @@ public class AmqpProvider implements Provider, TransportListener , AmqpResourceP
 
     @Override
     public void rollback(final JmsTransactionInfo transactionInfo, JmsTransactionInfo nextTransactionId, final AsyncResult request) throws IOException {
-        checkClosed();
+        checkClosedOrFailed();
         serializer.execute(new Runnable() {
 
             @Override
             public void run() {
                 try {
-                    checkClosed();
+                    checkClosedOrFailed();
                     AmqpSession session = connection.getSession(transactionInfo.getSessionId());
                     session.rollback(transactionInfo, nextTransactionId, request);
                     pumpToProtonTransport(request);
@@ -679,13 +730,13 @@ public class AmqpProvider implements Provider, TransportListener , AmqpResourceP
 
     @Override
     public void recover(final JmsSessionId sessionId, final AsyncResult request) throws IOException {
-        checkClosed();
+        checkClosedOrFailed();
         serializer.execute(new Runnable() {
 
             @Override
             public void run() {
                 try {
-                    checkClosed();
+                    checkClosedOrFailed();
                     AmqpSession session = connection.getSession(sessionId);
                     session.recover();
                     pumpToProtonTransport(request);
@@ -699,13 +750,13 @@ public class AmqpProvider implements Provider, TransportListener , AmqpResourceP
 
     @Override
     public void unsubscribe(final String subscription, final AsyncResult request) throws IOException {
-        checkClosed();
+        checkClosedOrFailed();
         serializer.execute(new Runnable() {
 
             @Override
             public void run() {
                 try {
-                    checkClosed();
+                    checkClosedOrFailed();
                     connection.unsubscribe(subscription, request);
                     pumpToProtonTransport(request);
                 } catch (Throwable t) {
@@ -717,22 +768,14 @@ public class AmqpProvider implements Provider, TransportListener , AmqpResourceP
 
     @Override
     public void pull(final JmsConsumerId consumerId, final long timeout, final AsyncResult request) throws IOException {
-        checkClosed();
+        checkClosedOrFailed();
         serializer.execute(new Runnable() {
 
             @Override
             public void run() {
                 try {
-                    checkClosed();
-                    AmqpConsumer consumer = null;
-
-                    if (consumerId.getProviderHint() instanceof AmqpConsumer) {
-                        consumer = (AmqpConsumer) consumerId.getProviderHint();
-                    } else {
-                        AmqpSession session = connection.getSession(consumerId.getParentId());
-                        consumer = session.getConsumer(consumerId);
-                    }
-
+                    checkClosedOrFailed();
+                    AmqpConsumer consumer = (AmqpConsumer) consumerId.getProviderHint();
                     consumer.pull(timeout, request);
                     pumpToProtonTransport(request);
                 } catch (Throwable t) {
@@ -764,7 +807,7 @@ public class AmqpProvider implements Provider, TransportListener , AmqpResourceP
     public void onData(final ByteBuf input) {
 
         // We need to retain until the serializer gets around to processing it.
-        ReferenceCountUtil.retain(input);
+        input.retain();
 
         serializer.execute(new Runnable() {
 
@@ -775,19 +818,16 @@ public class AmqpProvider implements Provider, TransportListener , AmqpResourceP
                         TRACE_BYTES.info("Received: {}", ByteBufUtil.hexDump(input));
                     }
 
-                    ByteBuffer source = input.nioBuffer();
-
                     do {
-                        ByteBuffer buffer = protonTransport.getInputBuffer();
-                        int limit = Math.min(buffer.remaining(), source.remaining());
-                        ByteBuffer duplicate = source.duplicate();
-                        duplicate.limit(source.position() + limit);
-                        buffer.put(duplicate);
-                        protonTransport.processInput().checkIsOk();
-                        source.position(source.position() + limit);
-                    } while (source.hasRemaining());
+                        ByteBuffer buffer = protonTransport.tail();
+                        int chunkSize = Math.min(buffer.remaining(), input.readableBytes());
+                        buffer.limit(buffer.position() + chunkSize);
+                        input.readBytes(buffer);
+                        protonTransport.process();
+                    } while (input.isReadable());
 
-                    ReferenceCountUtil.release(input);
+                    // Free for pooled memory to be put back now.
+                    input.release();
 
                     // Process the state changes from the latest data and then answer back
                     // any pending updates to the Broker.
@@ -795,7 +835,6 @@ public class AmqpProvider implements Provider, TransportListener , AmqpResourceP
                     pumpToProtonTransport();
                 } catch (Throwable t) {
                     LOG.warn("Caught problem during data processing: {}", t.getMessage(), t);
-
                     fireProviderException(t);
                 }
             }
@@ -868,34 +907,6 @@ public class AmqpProvider implements Provider, TransportListener , AmqpResourceP
         }
     }
 
-    @Override
-    public void onSaslMechanisms(Sasl sasl, org.apache.qpid.proton.engine.Transport transport) {
-        authenticator.handleSaslMechanisms(sasl, transport);
-        checkSaslAuthenticationState();
-    }
-
-    @Override
-    public void onSaslChallenge(Sasl sasl, org.apache.qpid.proton.engine.Transport transport) {
-        authenticator.handleSaslChallenge(sasl, transport);
-        checkSaslAuthenticationState();
-    }
-
-    @Override
-    public void onSaslOutcome(Sasl sasl, org.apache.qpid.proton.engine.Transport transport) {
-        authenticator.handleSaslOutcome(sasl, transport);
-        checkSaslAuthenticationState();
-    }
-
-    @Override
-    public void onSaslInit(Sasl sasl, org.apache.qpid.proton.engine.Transport transport) {
-        // Server only event
-    }
-
-    @Override
-    public void onSaslResponse(Sasl sasl, org.apache.qpid.proton.engine.Transport transport) {
-        // Server only event
-    }
-
     private void checkSaslAuthenticationState() {
         try {
             if (authenticator.isComplete()) {
@@ -1004,12 +1015,18 @@ public class AmqpProvider implements Provider, TransportListener , AmqpResourceP
     }
 
     protected boolean pumpToProtonTransport() {
-        return pumpToProtonTransport(NOOP_REQUEST);
+        return pumpToProtonTransport(NOOP_REQUEST, true);
     }
 
     protected boolean pumpToProtonTransport(AsyncResult request) {
+        return pumpToProtonTransport(request, true);
+    }
+
+    protected boolean pumpToProtonTransport(AsyncResult request, boolean flush) {
         try {
             boolean done = false;
+            int bytesWritten = 0;
+
             while (!done) {
                 ByteBuffer toWrite = protonTransport.getOutputBuffer();
                 if (toWrite != null && toWrite.hasRemaining()) {
@@ -1020,12 +1037,22 @@ public class AmqpProvider implements Provider, TransportListener , AmqpResourceP
                         TRACE_BYTES.info("Sending: {}", ByteBufUtil.hexDump(outbound));
                     }
 
-                    transport.send(outbound);
+                    bytesWritten += outbound.readableBytes();
+                    if (flush && bytesWritten >= getMaxWriteBytesBeforeFlush()) {
+                        transport.flush();
+                        bytesWritten = 0;
+                    }
+
+                    transport.write(outbound);
                     protonTransport.outputConsumed();
                 } else {
                     done = true;
                 }
             }
+
+            if (flush && bytesWritten > 0) {
+                transport.flush();
+            }
         } catch (IOException e) {
             fireProviderException(e);
             request.onFailure(e);
@@ -1072,6 +1099,8 @@ public class AmqpProvider implements Provider, TransportListener , AmqpResourceP
             nextIdleTimeoutCheck = null;
         }
 
+        failureCause = ex;
+
         ProviderListener listener = this.listener;
         if (listener != null) {
             listener.onConnectionFailure(IOExceptionSupport.create(ex));
@@ -1107,6 +1136,16 @@ public class AmqpProvider implements Provider, TransportListener , AmqpResourceP
         return connection.getAmqpMessageFactory();
     }
 
+    @Override
+    public ProviderFuture newProviderFuture() {
+        return futureFactory.createFuture();
+    }
+
+    @Override
+    public ProviderFuture newProviderFuture(ProviderSynchronization synchronization) {
+        return futureFactory.createFuture(synchronization);
+    }
+
     public void setTraceFrames(boolean trace) {
         this.traceFrames = trace;
         updateTracer();
@@ -1219,6 +1258,21 @@ public class AmqpProvider implements Provider, TransportListener , AmqpResourceP
         return maxFrameSize;
     }
 
+    public int getMaxWriteBytesBeforeFlush() {
+        return maxWriteBytesBeforeFlush;
+    }
+
+    /**
+     * Sets the maximum number of bytes that will be written on a large set of batched writes
+     * before a flush is requested on the {@link Transport}.
+     *
+     * @param maxWriteBytesBeforeFlush
+     * 		number of bytes written before a flush is requested.
+     */
+    public void setMaxWriteBytesBeforeFlush(int maxWriteBytesBeforeFlush) {
+        this.maxWriteBytesBeforeFlush = maxWriteBytesBeforeFlush;
+    }
+
     /**
      * Sets the max frame size (in bytes).
      *
@@ -1407,19 +1461,39 @@ public class AmqpProvider implements Provider, TransportListener , AmqpResourceP
 
     //----- Internal implementation ------------------------------------------//
 
-    private void checkClosed() throws ProviderClosedException {
+    private void checkClosedOrFailed() throws ProviderClosedException, ProviderFailedException {
         if (closed.get()) {
             throw new ProviderClosedException("This Provider is already closed");
         }
+
+        if (failureCause != null) {
+            throw new ProviderFailedException("The Provider has failed", failureCause);
+        }
     }
 
     private Mechanism findSaslMechanism(String[] remoteMechanisms) throws JMSSecurityRuntimeException {
 
-        Mechanism mechanism = SaslMechanismFinder.findMatchingMechanism(
-            connectionInfo.getUsername(), connectionInfo.getPassword(), transport.getLocalPrincipal(), saslMechanisms, remoteMechanisms);
+        final String username;
+        if (connectionInfo.getExtensionMap().containsKey(JmsConnectionExtensions.USERNAME_OVERRIDE)) {
+            username = (String) connectionInfo.getExtensionMap().get(
+                JmsConnectionExtensions.USERNAME_OVERRIDE).apply(connectionInfo.getConnection(), transport.getRemoteLocation());
+        } else {
+            username = connectionInfo.getUsername();
+        }
 
-        mechanism.setUsername(connectionInfo.getUsername());
-        mechanism.setPassword(connectionInfo.getPassword());
+        final String password;
+        if (connectionInfo.getExtensionMap().containsKey(JmsConnectionExtensions.PASSWORD_OVERRIDE)) {
+            password = (String) connectionInfo.getExtensionMap().get(
+                JmsConnectionExtensions.PASSWORD_OVERRIDE).apply(connectionInfo.getConnection(), transport.getRemoteLocation());
+        } else {
+            password = connectionInfo.getPassword();
+        }
+
+        Mechanism mechanism = SaslMechanismFinder.findMatchingMechanism(
+            username, password, transport.getLocalPrincipal(), saslMechanisms, remoteMechanisms);
+
+        mechanism.setUsername(username);
+        mechanism.setPassword(password);
 
         try {
             Map<String, String> saslOptions = PropertyUtil.filterProperties(PropertyUtil.parseQuery(getRemoteURI()), "sasl.options.");
