@@ -20,6 +20,8 @@ import java.io.IOException;
 import java.net.URI;
 import java.util.ArrayList;
 import java.util.Arrays;
+import java.util.Collections;
+import java.util.ConcurrentModificationException;
 import java.util.LinkedHashMap;
 import java.util.List;
 import java.util.Locale;
@@ -51,8 +53,10 @@ import org.apache.qpid.jms.provider.Provider;
 import org.apache.qpid.jms.provider.ProviderConstants.ACK_TYPE;
 import org.apache.qpid.jms.provider.ProviderFactory;
 import org.apache.qpid.jms.provider.ProviderFuture;
+import org.apache.qpid.jms.provider.ProviderFutureFactory;
 import org.apache.qpid.jms.provider.ProviderListener;
 import org.apache.qpid.jms.provider.ProviderRedirectedException;
+import org.apache.qpid.jms.provider.ProviderSynchronization;
 import org.apache.qpid.jms.provider.WrappedAsyncResult;
 import org.apache.qpid.jms.util.IOExceptionSupport;
 import org.apache.qpid.jms.util.QpidJMSThreadFactory;
@@ -96,15 +100,13 @@ public class FailoverProvider extends DefaultProviderListener implements Provide
     private final AtomicBoolean failed = new AtomicBoolean();
     private final AtomicBoolean closingConnection = new AtomicBoolean(false);
     private final AtomicLong requestId = new AtomicLong();
-    private final Map<Long, FailoverRequest> requests = new LinkedHashMap<Long, FailoverRequest>();
+    private final Map<Long, FailoverRequest> requests = Collections.synchronizedMap(new LinkedHashMap<Long, FailoverRequest>());
     private final DefaultProviderListener closedListener = new DefaultProviderListener();
     private final AtomicReference<JmsMessageFactory> messageFactory = new AtomicReference<JmsMessageFactory>();
+    private final ProviderFutureFactory futureFactory;
 
     // Current state of connection / reconnection
-    private boolean firstAttempt = true;
-    private boolean firstConnection = true;
-    private long reconnectAttempts;
-    private long nextReconnectDelay = -1;
+    private final ReconnectControls reconnectControl = new ReconnectControls();
     private IOException failureCause;
     private URI connectedURI;
     private volatile JmsConnectionInfo connectionInfo;
@@ -126,16 +128,17 @@ public class FailoverProvider extends DefaultProviderListener implements Provide
 
     private FailoverServerListAction amqpOpenServerListAction = FailoverServerListAction.REPLACE;
 
-    public FailoverProvider(Map<String, String> nestedOptions) {
-        this(null, nestedOptions);
+    public FailoverProvider(Map<String, String> nestedOptions, ProviderFutureFactory futureFactory) {
+        this(null, nestedOptions, futureFactory);
     }
 
-    public FailoverProvider(List<URI> uris) {
-        this(uris, null);
+    public FailoverProvider(List<URI> uris, ProviderFutureFactory futureFactory) {
+        this(uris, null, futureFactory);
     }
 
-    public FailoverProvider(List<URI> uris, Map<String, String> nestedOptions) {
+    public FailoverProvider(List<URI> uris, Map<String, String> nestedOptions, ProviderFutureFactory futureFactory) {
         this.uris = new FailoverUriPool(uris, nestedOptions);
+        this.futureFactory = futureFactory;
 
         serializer = new ScheduledThreadPoolExecutor(1, new QpidJMSThreadFactory("FailoverProvider: serialization thread", true));
         serializer.setExecuteExistingDelayedTasksAfterShutdownPolicy(false);
@@ -169,16 +172,21 @@ public class FailoverProvider extends DefaultProviderListener implements Provide
     @Override
     public void close() {
         if (closed.compareAndSet(false, true)) {
-            final ProviderFuture request = new ProviderFuture();
+            final ProviderFuture request = futureFactory.createFuture();
             serializer.execute(new Runnable() {
 
                 @Override
                 public void run() {
                     try {
                         IOException error = failureCause != null ? failureCause : new IOException("Connection closed");
-                        List<FailoverRequest> pending = new ArrayList<FailoverRequest>(requests.values());
+                        final List<FailoverRequest> pending;
+                        synchronized (requests) {
+                            pending = new ArrayList<FailoverRequest>(requests.values());
+                        }
                         for (FailoverRequest request : pending) {
-                            request.onFailure(error);
+                            if (!request.isComplete()) {
+                                request.onFailure(error);
+                            }
                         }
 
                         if (requestTimeoutTask != null) {
@@ -542,8 +550,7 @@ public class FailoverProvider extends DefaultProviderListener implements Provide
             }
             provider = null;
 
-            if (reconnectAllowed(cause)) {
-
+            if (reconnectControl.isReconnectAllowed(cause)) {
                 if (cause instanceof ProviderRedirectedException) {
                     ProviderRedirectedException redirect = (ProviderRedirectedException) cause;
                     try {
@@ -558,6 +565,14 @@ public class FailoverProvider extends DefaultProviderListener implements Provide
                     listener.onConnectionInterrupted(failedURI);
                 }
 
+                final List<FailoverRequest> pending;
+                synchronized (requests) {
+                    pending = new ArrayList<FailoverRequest>(requests.values());
+                }
+                for (FailoverRequest request : pending) {
+                    request.whenOffline(cause);
+                }
+
                 // Start watching for request timeouts while we are offline, unless we already are.
                 if (requestTimeoutTask == null) {
                     long sweeperInterval = getRequestSweeperInterval();
@@ -596,7 +611,7 @@ public class FailoverProvider extends DefaultProviderListener implements Provide
                     provider.setProviderListener(FailoverProvider.this);
                     connectedURI = provider.getRemoteURI();
 
-                    if (!firstConnection) {
+                    if (reconnectControl.isRecoveryRequired()) {
                         LOG.debug("Signalling connection recovery: {}", provider);
 
                         // Stage 1: Allow listener to recover its resources
@@ -617,20 +632,34 @@ public class FailoverProvider extends DefaultProviderListener implements Provide
 
                         // Stage 5: Let the client know that connection has restored.
                         listener.onConnectionRestored(provider.getRemoteURI());
+
+                        // Last step: Send pending actions.
+                        final List<FailoverRequest> pending;
+                        synchronized (requests) {
+                            pending = new ArrayList<FailoverRequest>(requests.values());
+                        }
+                        for (FailoverRequest request : pending) {
+                            if (!request.isComplete()) {
+                                request.run();
+                            }
+                        }
+
+                        reconnectControl.connectionEstablished();
                     } else {
                         processAlternates(provider.getAlternateURIs());
-                    }
 
-                    // Last step: Send pending actions.
-                    List<FailoverRequest> pending = new ArrayList<FailoverRequest>(requests.values());
-                    for (FailoverRequest request : pending) {
-                        request.run();
+                        // Last step: Send pending actions.
+                        final List<FailoverRequest> pending;
+                        synchronized (requests) {
+                            pending = new ArrayList<FailoverRequest>(requests.values());
+                        }
+                        for (FailoverRequest request : pending) {
+                            if (!request.isComplete()) {
+                                request.run();
+                            }
+                        }
                     }
 
-                    nextReconnectDelay = reconnectDelay;
-                    reconnectAttempts = 0;
-                    uris.connected();
-
                     // Cancel timeout processing since we are connected again.  We waited until
                     // now for the case where we are continually getting bounced from otherwise
                     // live servers, we want the timeout to remain scheduled in that case so that
@@ -641,6 +670,7 @@ public class FailoverProvider extends DefaultProviderListener implements Provide
                     }
 
                 } catch (Throwable error) {
+                    LOG.trace("Connection attempt:[{}] to: {} failed", reconnectControl.reconnectAttempts, provider.getRemoteURI());
                     handleProviderFailure(IOExceptionSupport.create(error));
                 }
             }
@@ -649,8 +679,9 @@ public class FailoverProvider extends DefaultProviderListener implements Provide
 
     /**
      * Called when the Provider was either first created or when a connection failure has
-     * been reported.  A reconnection attempt is immediately executed on the connection
-     * thread.  If a new Provider is able to be created and connected then a recovery task
+     * been reported.  A reconnection attempt is executed on the connection thread either
+     * immediately or after a delay based on configuration an number of attempts that have
+     * elapsed.  If a new Provider is able to be created and connected then a recovery task
      * is scheduled on the main serializer thread.  If the connect attempt fails another
      * attempt is scheduled based on the configured delay settings until a max attempts
      * limit is hit, if one is set.
@@ -665,8 +696,7 @@ public class FailoverProvider extends DefaultProviderListener implements Provide
             return;
         }
 
-        connectionHub.execute(new Runnable() {
-            boolean delayed = false;
+        reconnectControl.scheduleReconnect(new Runnable() {
 
             @Override
             public void run() {
@@ -674,116 +704,87 @@ public class FailoverProvider extends DefaultProviderListener implements Provide
                     return;
                 }
 
-                int reconnectLimit = reconnectAttemptLimit();
-                if (reconnectLimit != UNLIMITED && reconnectAttempts >= reconnectLimit) {
-                    return;
-                }
-
-                boolean first = firstAttempt;
-                firstAttempt = false;
-
-                if (!delayed && !first && initialReconnectDelay > 0 && reconnectAttempts == 0) {
-                    delayed = true;
-                    LOG.trace("Delayed initial reconnect attempt will be in {} milliseconds", initialReconnectDelay);
-                    connectionHub.schedule(this, initialReconnectDelay, TimeUnit.MILLISECONDS);
-                    return;
-                }
-
-                reconnectAttempts++;
                 Throwable failure = null;
-                if (!uris.isEmpty()) {
-                    for (int i = 0; i < uris.size(); ++i) {
-                        URI target = uris.getNext();
-                        if (target == null) {
-                            LOG.warn("Failover URI collection unexpectedly modified during connection attempt.");
-                            continue;
-                        }
+                Provider provider = null;
+
+                long reconnectAttempts = reconnectControl.recordNextAttempt();
+
+                try {
+                    if (!uris.isEmpty()) {
+                        for (int i = 0; i < uris.size(); ++i) {
+                            URI target = uris.getNext();
+                            if (target == null) {
+                                LOG.trace("Failover URI collection unexpectedly modified during connection attempt.");
+                                failure = new ConcurrentModificationException("Failover URIs changed unexpectedly");
+                                continue;
+                            }
 
-                        Provider provider = null;
-                        try {
-                            LOG.debug("Connection attempt:[{}] to: {} in-progress", reconnectAttempts, target);
-                            provider = ProviderFactory.create(target);
-                            provider.connect(connectionInfo);
-                            initializeNewConnection(provider);
-                            return;
-                        } catch (Throwable e) {
-                            LOG.info("Connection attempt:[{}] to: {} failed", reconnectAttempts, target);
-                            failure = e;
                             try {
-                                if (provider != null) {
-                                    provider.close();
+                                LOG.debug("Connection attempt:[{}] to: {} in-progress", reconnectAttempts,
+                                    target.getScheme() + "://" + target.getHost() + ":" + target.getPort());
+                                provider = ProviderFactory.create(target, futureFactory);
+                                provider.connect(connectionInfo);
+                                initializeNewConnection(provider);
+                                return;
+                            } catch (Throwable e) {
+                                LOG.info("Connection attempt:[{}] to: {} failed", reconnectAttempts,
+                                    target.getScheme() + "://" + target.getHost() + ":" + target.getPort());
+                                failure = e;
+                                try {
+                                    if (provider != null) {
+                                        provider.close();
+                                    }
+                                } catch (Throwable ex) {
+                                } finally {
+                                    provider = null;
                                 }
-                            } catch (Throwable ex) {}
+                            }
+                        }
+                    } else {
+                        LOG.debug("No remote URI available to connect to in failover list");
+                        failure = new IOException(
+                            "No remote URI available for reconnection during connection attempt: " + reconnectAttempts);
+                    }
+                } catch (Throwable unknownFailure) {
+                    LOG.warn("Connection attempt:[{}] failed abnormally.", reconnectAttempts);
+                    failure = failure == null ? unknownFailure : failure;
+                } finally {
+                    if (provider == null) {
+                        LOG.trace("Connection attempt:[{}] failed error: {}", reconnectControl.reconnectAttempts, failure.getMessage());
+                        if (reconnectControl.isLimitExceeded()) {
+                            reportReconnectFailure(failure);
+                        } else {
+                            reconnectControl.scheduleReconnect(this);
                         }
                     }
-                } else {
-                    LOG.debug("No remote URI available to connect to in failover list");
                 }
-
-                if (reconnectLimit != UNLIMITED && reconnectAttempts >= reconnectLimit) {
-                    LOG.error("Failed to connect after: " + reconnectAttempts + " attempt(s)");
-                    failed.set(true);
-                    if (failure == null) {
-                        failureCause = new IOException("Failed to connect after: " + reconnectAttempts + " attempt(s)");
-                    } else {
-                        failureCause = IOExceptionSupport.create(failure);
-                    }
-                    if (listener != null) {
-                        listener.onConnectionFailure(failureCause);
-                    };
-
-                    return;
-                }
-
-                int warnInterval = getWarnAfterReconnectAttempts();
-                if (warnInterval > 0 && (reconnectAttempts % warnInterval) == 0) {
-                    LOG.warn("Failed to connect after: {} attempt(s) continuing to retry.", reconnectAttempts);
-                }
-
-                long delay = nextReconnectDelay();
-                LOG.trace("Next reconnect attempt will be in {} milliseconds", delay);
-                connectionHub.schedule(this, delay, TimeUnit.MILLISECONDS);
             }
         });
     }
 
-    private boolean reconnectAllowed(IOException cause) {
-        // If a connection attempts fail due to Security errors than
-        // we abort reconnection as there is a configuration issue and
-        // we want to avoid a spinning reconnect cycle that can never
-        // complete.
-        if (cause.getCause() instanceof JMSSecurityException) {
-            return false;
-        }
-
-        return reconnectAttemptLimit() != 0;
-    }
-
-    private int reconnectAttemptLimit() {
-        int maxReconnectValue = this.maxReconnectAttempts;
-        if (firstConnection && this.startupMaxReconnectAttempts != UNDEFINED) {
-            // If this is the first connection and a specific startup retry limit
-            // is configured then use it, otherwise use the main reconnect limit
-            maxReconnectValue = this.startupMaxReconnectAttempts;
-        }
-        return maxReconnectValue;
-    }
-
-    private long nextReconnectDelay() {
-
-        if (nextReconnectDelay == -1) {
-            nextReconnectDelay = reconnectDelay;
-        }
-
-        if (isUseReconnectBackOff() && reconnectAttempts > 1) {
-            // Exponential increment of reconnect delay.
-            nextReconnectDelay *= getReconnectBackOffMultiplier();
-            if (nextReconnectDelay > maxReconnectDelay) {
-                nextReconnectDelay = maxReconnectDelay;
+    /**
+     * Called when the reconnection executor has tried for the last time based on max reconnects
+     * configuration and we now consider this connection attempt to be failed.  This method will
+     * run the failure processing on the serializer executor to ensure that all events back to the
+     * client layer happen from the same thread.
+     *
+     * @param lastFailure the last failure encountered while trying to (re)connect.
+     */
+    private void reportReconnectFailure(final Throwable lastFailure) {
+        serializer.execute(() -> {
+            LOG.error("Failed to connect after: " + reconnectControl.reconnectAttempts + " attempt(s)");
+            if (failed.compareAndSet(false, true)) {
+                if (lastFailure == null) {
+                    failureCause = new IOException(
+                        "Failed to connect after: " + reconnectControl.reconnectAttempts + " attempt(s)");
+                } else {
+                    failureCause = IOExceptionSupport.create(lastFailure);
+                }
+                if (listener != null) {
+                    listener.onConnectionFailure(failureCause);
+                };
             }
-        }
-
-        return nextReconnectDelay;
+        });
     }
 
     protected void checkClosed() throws IOException {
@@ -837,6 +838,15 @@ public class FailoverProvider extends DefaultProviderListener implements Provide
         });
     }
 
+    @Override
+    public void onResourceClosed(JmsResource resource, Throwable cause) {
+        if (closingConnection.get() || closed.get() || failed.get()) {
+            return;
+        }
+
+        listener.onResourceClosed(resource, cause);
+    }
+
     @Override
     public void onProviderException(final Exception ex) {
         if (closingConnection.get() || closed.get() || failed.get()) {
@@ -853,9 +863,14 @@ public class FailoverProvider extends DefaultProviderListener implements Provide
         });
     }
 
+    //--------------- Processing for server-provided alternate URIs  ---------//
+
     private void processAlternates(List<URI> alternates) {
         if (!alternates.isEmpty()) {
             List<URI> newRemotes = new ArrayList<URI>(alternates);
+
+            LOG.debug("Processing alternates uris:{} with new set: {}", uris, newRemotes);
+
             switch (amqpOpenServerListAction) {
                 case ADD:
                     try {
@@ -882,6 +897,8 @@ public class FailoverProvider extends DefaultProviderListener implements Provide
                     // Shouldn't get here, but do nothing if we do.
                     break;
             }
+
+            LOG.debug("Processing alternates done new uris:{}", uris);
         }
     }
 
@@ -1049,6 +1066,16 @@ public class FailoverProvider extends DefaultProviderListener implements Provide
         return uris.getNestedOptions();
     }
 
+    @Override
+    public ProviderFuture newProviderFuture() {
+        return futureFactory.createFuture();
+    }
+
+    @Override
+    public ProviderFuture newProviderFuture(ProviderSynchronization synchronization) {
+        return futureFactory.createFuture(synchronization);
+    }
+
     @Override
     public String toString() {
         return "FailoverProvider: " +
@@ -1075,8 +1102,11 @@ public class FailoverProvider extends DefaultProviderListener implements Provide
 
         @Override
         public void run() {
-            List<FailoverRequest> copied = new ArrayList<FailoverRequest>(requests.values());
-            for (FailoverRequest request : copied) {
+            final List<FailoverRequest> pending;
+            synchronized (requests) {
+                pending = new ArrayList<FailoverRequest>(requests.values());
+            }
+            for (FailoverRequest request : pending) {
                 if (request.isExpired()) {
                     LOG.trace("Task {} has timed out, sending failure notice.", request);
                     request.onFailure(request.createTimedOutException());
@@ -1229,21 +1259,13 @@ public class FailoverProvider extends DefaultProviderListener implements Provide
 
         @Override
         public void onSuccess() {
-            serializer.execute(new Runnable() {
-                @Override
-                public void run() {
-                    if (firstConnection) {
-                        LOG.trace("First connection requst has completed:");
-                        FailoverProvider.this.messageFactory.set(provider.getMessageFactory());
-                        processAlternates(provider.getAlternateURIs());
-                        listener.onConnectionEstablished(provider.getRemoteURI());
-                        firstConnection = false;
-                    } else {
-                        LOG.warn("A second call to a CreateConnectionRequest not expected.");
-                    }
-
-                    CreateConnectionRequest.this.signalConnected();
-                }
+            serializer.execute(() -> {
+                LOG.trace("First connection requst has completed:");
+                FailoverProvider.this.messageFactory.set(provider.getMessageFactory());
+                processAlternates(provider.getAlternateURIs());
+                listener.onConnectionEstablished(provider.getRemoteURI());
+                reconnectControl.connectionEstablished();
+                CreateConnectionRequest.this.signalConnected();
             });
         }
 
@@ -1254,14 +1276,11 @@ public class FailoverProvider extends DefaultProviderListener implements Provide
                 super.onFailure(result);
             } else {
                 LOG.debug("Request received error: {}", result.getMessage());
-                serializer.execute(new Runnable() {
-                    @Override
-                    public void run() {
-                        // If we managed to receive an Open frame it might contain
-                        // a failover update so process it before handling the error.
-                        processAlternates(provider.getAlternateURIs());
-                        handleProviderFailure(IOExceptionSupport.create(result));
-                    }
+                serializer.execute(() -> {
+                    // If we managed to receive an Open frame it might contain
+                    // a failover update so process it before handling the error.
+                    processAlternates(provider.getAlternateURIs());
+                    handleProviderFailure(IOExceptionSupport.create(result));
                 });
             }
         }
@@ -1274,4 +1293,115 @@ public class FailoverProvider extends DefaultProviderListener implements Provide
     private static enum FailoverServerListAction {
         ADD, REPLACE, IGNORE
     }
+
+    private class ReconnectControls {
+
+        // Reconnection state tracking
+        private volatile boolean recoveryRequired;
+        private volatile long reconnectAttempts;
+        private volatile long nextReconnectDelay = -1;
+
+        public void scheduleReconnect(Runnable runnable) {
+            try {
+                // Warn of ongoing connection attempts if configured.
+                int warnInterval = getWarnAfterReconnectAttempts();
+                if (reconnectAttempts > 0 && warnInterval > 0 && (reconnectAttempts % warnInterval) == 0) {
+                    LOG.warn("Failed to connect after: {} attempt(s) continuing to retry.", reconnectAttempts);
+                }
+
+                // If no connection recovery required then we have never fully connected to a remote
+                // so we proceed down the connect with one immediate connection attempt and then follow
+                // on delayed attempts based on configuration.
+                if (!recoveryRequired) {
+                    if (reconnectAttempts == 0) {
+                        LOG.trace("Initial connect attempt will be performed immediately");
+                        connectionHub.execute(runnable);
+                    } else if (reconnectAttempts == 1 && initialReconnectDelay > 0) {
+                        LOG.trace("Delayed initial reconnect attempt will be in {} milliseconds", initialReconnectDelay);
+                        connectionHub.schedule(runnable, initialReconnectDelay, TimeUnit.MILLISECONDS);
+                    } else {
+                        long delay = reconnectControl.nextReconnectDelay();
+                        LOG.trace("Next reconnect attempt will be in {} milliseconds", delay);
+                        connectionHub.schedule(runnable, delay, TimeUnit.MILLISECONDS);
+                    }
+                } else if (reconnectAttempts == 0) {
+                    if (initialReconnectDelay > 0) {
+                        LOG.trace("Delayed initial reconnect attempt will be in {} milliseconds", initialReconnectDelay);
+                        connectionHub.schedule(runnable, initialReconnectDelay, TimeUnit.MILLISECONDS);
+                    } else {
+                        LOG.trace("Initial Reconnect attempt will be performed immediately");
+                        connectionHub.execute(runnable);
+                    }
+                } else {
+                    long delay = reconnectControl.nextReconnectDelay();
+                    LOG.trace("Next reconnect attempt will be in {} milliseconds", delay);
+                    connectionHub.schedule(runnable, delay, TimeUnit.MILLISECONDS);
+                }
+            } catch (Throwable unrecoverable) {
+                reportReconnectFailure(unrecoverable);
+            }
+        }
+
+        public void connectionEstablished() {
+            recoveryRequired = true;
+            nextReconnectDelay = -1;
+            reconnectAttempts = 0;
+            uris.connected();
+        }
+
+        public long recordNextAttempt() {
+            return ++reconnectAttempts;
+        }
+
+        public boolean isRecoveryRequired() {
+            return recoveryRequired;
+        }
+
+        public boolean isLimitExceeded() {
+            int reconnectLimit = reconnectAttemptLimit();
+            if (reconnectLimit != UNLIMITED && reconnectAttempts >= reconnectLimit) {
+                return true;
+            }
+
+            return false;
+        }
+
+        public boolean isReconnectAllowed(IOException cause) {
+            // If a connection attempts fail due to Security errors than
+            // we abort reconnection as there is a configuration issue and
+            // we want to avoid a spinning reconnect cycle that can never
+            // complete.
+            if (cause.getCause() instanceof JMSSecurityException) {
+                return false;
+            }
+
+            return !isLimitExceeded();
+        }
+
+        private int reconnectAttemptLimit() {
+            int maxReconnectValue = maxReconnectAttempts;
+            if (!recoveryRequired && startupMaxReconnectAttempts != UNDEFINED) {
+                // If this is the first connection attempt and a specific startup retry limit
+                // is configured then use it, otherwise use the main reconnect limit
+                maxReconnectValue = startupMaxReconnectAttempts;
+            }
+            return maxReconnectValue;
+        }
+
+        private long nextReconnectDelay() {
+            if (nextReconnectDelay == -1) {
+                nextReconnectDelay = reconnectDelay;
+            }
+
+            if (isUseReconnectBackOff() && reconnectAttempts > 1) {
+                // Exponential increment of reconnect delay.
+                nextReconnectDelay *= getReconnectBackOffMultiplier();
+                if (nextReconnectDelay > maxReconnectDelay) {
+                    nextReconnectDelay = maxReconnectDelay;
+                }
+            }
+
+            return nextReconnectDelay;
+        }
+    }
 }
