@@ -26,8 +26,12 @@ import static org.junit.Assert.assertNotNull;
 import static org.junit.Assert.assertNull;
 import static org.junit.Assert.fail;
 
+import java.net.URI;
 import java.net.URLDecoder;
 import java.net.URLEncoder;
+import java.nio.charset.StandardCharsets;
+import java.util.Base64;
+import java.util.concurrent.atomic.AtomicReference;
 
 import javax.jms.Connection;
 import javax.jms.ConnectionFactory;
@@ -36,10 +40,11 @@ import javax.jms.JMSSecurityException;
 import javax.jms.JMSSecurityRuntimeException;
 import javax.net.ssl.SSLContext;
 
+import org.apache.qpid.jms.JmsConnectionExtensions;
 import org.apache.qpid.jms.JmsConnectionFactory;
 import org.apache.qpid.jms.test.QpidJmsTestCase;
 import org.apache.qpid.jms.test.testpeer.TestAmqpPeer;
-import org.apache.qpid.jms.transports.TransportSslOptions;
+import org.apache.qpid.jms.transports.TransportOptions;
 import org.apache.qpid.jms.transports.TransportSupport;
 import org.apache.qpid.proton.amqp.Symbol;
 import org.junit.Test;
@@ -56,6 +61,7 @@ public class SaslIntegrationTest extends QpidJmsTestCase {
     private static final Symbol SCRAM_SHA_1 = Symbol.valueOf("SCRAM-SHA-1");
     private static final Symbol SCRAM_SHA_256 = Symbol.valueOf("SCRAM-SHA-256");
     private static final Symbol EXTERNAL = Symbol.valueOf("EXTERNAL");
+    private static final Symbol XOAUTH2 = Symbol.valueOf("XOAUTH2");
 
     private static final String BROKER_JKS_KEYSTORE = "src/test/resources/broker-jks.keystore";
     private static final String BROKER_JKS_TRUSTSTORE = "src/test/resources/broker-jks.truststore";
@@ -65,7 +71,7 @@ public class SaslIntegrationTest extends QpidJmsTestCase {
 
     @Test(timeout = 20000)
     public void testSaslExternalConnection() throws Exception {
-        TransportSslOptions sslOptions = new TransportSslOptions();
+        TransportOptions sslOptions = new TransportOptions();
         sslOptions.setKeyStoreLocation(BROKER_JKS_KEYSTORE);
         sslOptions.setKeyStorePassword(PASSWORD);
         sslOptions.setVerifyHost(false);
@@ -77,7 +83,7 @@ public class SaslIntegrationTest extends QpidJmsTestCase {
                              "transport.keyStoreLocation=" + CLIENT_JKS_KEYSTORE + "&" +
                              "transport.keyStorePassword=" + PASSWORD;
 
-        SSLContext context = TransportSupport.createSslContext(sslOptions);
+        SSLContext context = TransportSupport.createJdkSslContext(sslOptions);
 
         try (TestAmqpPeer testPeer = new TestAmqpPeer(context, true);) {
             // Expect an EXTERNAL connection
@@ -127,6 +133,33 @@ public class SaslIntegrationTest extends QpidJmsTestCase {
         }
     }
 
+    @Test(timeout = 20000)
+    public void testSaslXOauth2Connection() throws Exception {
+        try (TestAmqpPeer testPeer = new TestAmqpPeer();) {
+
+            // Expect a XOAUTH2 connection
+            String user = "user";
+            String pass = "eyB1c2VyPSJ1c2VyIiB9";
+
+            testPeer.expectSaslXOauth2(user, pass);
+            testPeer.expectOpen();
+
+            // Each connection creates a session for managing temporary destinations etc
+            testPeer.expectBegin();
+
+            ConnectionFactory factory = new JmsConnectionFactory("amqp://localhost:" + testPeer.getServerPort());
+            Connection connection = factory.createConnection(user, pass);
+            // Set a clientID to provoke the actual AMQP connection process to occur.
+            connection.setClientID("clientName");
+
+            testPeer.waitForAllHandlersToComplete(1000);
+            assertNull(testPeer.getThrowable());
+
+            testPeer.expectClose();
+            connection.close();
+        }
+    }
+
     @Test(timeout = 20000)
     public void testSaslPlainConnectionWithURIEncodedCredentials() throws Exception {
         try (TestAmqpPeer testPeer = new TestAmqpPeer();) {
@@ -228,6 +261,12 @@ public class SaslIntegrationTest extends QpidJmsTestCase {
         doMechanismSelectedTestImpl("username", "password", SCRAM_SHA_256, new Symbol[] {SCRAM_SHA_256, SCRAM_SHA_1, CRAM_MD5, PLAIN, ANONYMOUS}, false);
     }
 
+    @Test(timeout = 20000)
+    public void testXoauth2SelectedWhenCredentialsPresent() throws Exception {
+        String token = Base64.getEncoder().encodeToString("token".getBytes(StandardCharsets.US_ASCII));
+        doMechanismSelectedTestImpl("username", token, XOAUTH2, new Symbol[] {XOAUTH2, ANONYMOUS}, false);
+    }
+
     private void doMechanismSelectedTestImpl(String username, String password, Symbol clientSelectedMech, Symbol[] serverMechs, boolean wait) throws Exception {
         try (TestAmqpPeer testPeer = new TestAmqpPeer();) {
 
@@ -264,7 +303,7 @@ public class SaslIntegrationTest extends QpidJmsTestCase {
     }
 
     private void doMechanismSelectedExternalTestImpl(boolean requireClientCert, Symbol clientSelectedMech, Symbol[] serverMechs) throws Exception {
-        TransportSslOptions sslOptions = new TransportSslOptions();
+        TransportOptions sslOptions = new TransportOptions();
         sslOptions.setKeyStoreLocation(BROKER_JKS_KEYSTORE);
         sslOptions.setKeyStorePassword(PASSWORD);
         sslOptions.setVerifyHost(false);
@@ -273,7 +312,7 @@ public class SaslIntegrationTest extends QpidJmsTestCase {
             sslOptions.setTrustStorePassword(PASSWORD);
         }
 
-        SSLContext context = TransportSupport.createSslContext(sslOptions);
+        SSLContext context = TransportSupport.createJdkSslContext(sslOptions);
 
         try (TestAmqpPeer testPeer = new TestAmqpPeer(context, requireClientCert);) {
             String connOptions = "?transport.trustStoreLocation=" + CLIENT_JKS_TRUSTSTORE + "&" +
@@ -412,4 +451,121 @@ public class SaslIntegrationTest extends QpidJmsTestCase {
             testPeer.waitForAllHandlersToComplete(1000);
         }
     }
+
+    @Test(timeout = 20000)
+    public void testUserOnlyExtensionsApplied() throws Exception {
+        try (TestAmqpPeer testPeer = new TestAmqpPeer();) {
+
+            final AtomicReference<Connection> connectionRef = new AtomicReference<>();
+            final AtomicReference<URI> remoteURIRef = new AtomicReference<>();
+
+            // Expect a PLAIN connection
+            final String user = "user";
+            final String pass = "qwerty123456";
+
+            testPeer.expectSaslPlain(user, pass);
+            testPeer.expectOpen();
+
+            // Each connection creates a session for managing temporary destinations etc
+            testPeer.expectBegin();
+
+            final URI remoteURI = new URI("amqp://localhost:" + testPeer.getServerPort());
+
+            JmsConnectionFactory factory = new JmsConnectionFactory(remoteURI);
+
+            factory.setExtension(JmsConnectionExtensions.USERNAME_OVERRIDE.toString(), (connection, uri) -> {
+                connectionRef.set(connection);
+                remoteURIRef.set(uri);
+
+                return user;
+            });
+
+            Connection connection = factory.createConnection(null, pass);
+            // Set a clientID to provoke the actual AMQP connection process to occur.
+            connection.setClientID("clientName");
+
+            testPeer.waitForAllHandlersToComplete(1000);
+            assertNull(testPeer.getThrowable());
+
+            assertEquals(connection, connectionRef.get());
+            assertEquals(remoteURI, remoteURIRef.get());
+
+            testPeer.expectClose();
+            connection.close();
+        }
+    }
+
+    @Test(timeout = 20000)
+    public void testPasswordOnlyExtensionsApplied() throws Exception {
+        try (TestAmqpPeer testPeer = new TestAmqpPeer();) {
+
+            final AtomicReference<Connection> connectionRef = new AtomicReference<>();
+            final AtomicReference<URI> remoteURIRef = new AtomicReference<>();
+
+            // Expect a PLAIN connection
+            final String user = "user";
+            final String pass = "qwerty123456";
+
+            testPeer.expectSaslPlain(user, pass);
+            testPeer.expectOpen();
+
+            // Each connection creates a session for managing temporary destinations etc
+            testPeer.expectBegin();
+
+            final URI remoteURI = new URI("amqp://localhost:" + testPeer.getServerPort());
+
+            JmsConnectionFactory factory = new JmsConnectionFactory(remoteURI);
+
+            factory.setExtension(JmsConnectionExtensions.PASSWORD_OVERRIDE.toString(), (connection, uri) -> {
+                connectionRef.set(connection);
+                remoteURIRef.set(uri);
+
+                return pass;
+            });
+
+            Connection connection = factory.createConnection(user, null);
+            // Set a clientID to provoke the actual AMQP connection process to occur.
+            connection.setClientID("clientName");
+
+            testPeer.waitForAllHandlersToComplete(1000);
+            assertNull(testPeer.getThrowable());
+
+            assertEquals(connection, connectionRef.get());
+            assertEquals(remoteURI, remoteURIRef.get());
+
+            testPeer.expectClose();
+            connection.close();
+        }
+    }
+
+    @Test(timeout = 20000)
+    public void testUserAndPasswordExtensionsApplied() throws Exception {
+        try (TestAmqpPeer testPeer = new TestAmqpPeer();) {
+
+            // Expect a PLAIN connection
+            final String user = "user";
+            final String pass = "qwerty123456";
+
+            testPeer.expectSaslPlain(user, pass);
+            testPeer.expectOpen();
+
+            // Each connection creates a session for managing temporary destinations etc
+            testPeer.expectBegin();
+
+            JmsConnectionFactory factory = new JmsConnectionFactory("amqp://localhost:" + testPeer.getServerPort());
+
+            factory.setExtension(JmsConnectionExtensions.USERNAME_OVERRIDE.toString(), (connection, uri) -> { return user; });
+            factory.setExtension(JmsConnectionExtensions.PASSWORD_OVERRIDE.toString(), (connection, uri) -> { return pass; });
+
+            Connection connection = factory.createConnection();
+            // Set a clientID to provoke the actual AMQP connection process to occur.
+            connection.setClientID("clientName");
+
+            testPeer.waitForAllHandlersToComplete(1000);
+            assertNull(testPeer.getThrowable());
+
+            testPeer.expectClose();
+            connection.close();
+        }
+    }
 }
