@@ -1,4 +1,4 @@
-/**
+/*
  * Licensed to the Apache Software Foundation (ASF) under one or more
  * contributor license agreements.  See the NOTICE file distributed with
  * this work for additional information regarding copyright ownership.
@@ -20,6 +20,7 @@ import static org.apache.qpid.jms.provider.amqp.message.AmqpMessageSupport.JMS_M
 import static org.apache.qpid.jms.provider.amqp.message.AmqpMessageSupport.JMS_TEXT_MESSAGE;
 import static org.apache.qpid.jms.provider.amqp.message.AmqpMessageSupport.getSymbol;
 import static org.junit.Assert.assertEquals;
+import static org.junit.Assert.assertFalse;
 import static org.junit.Assert.assertNotNull;
 import static org.junit.Assert.assertNull;
 import static org.junit.Assert.assertTrue;
@@ -37,8 +38,6 @@ import org.apache.qpid.proton.amqp.Symbol;
 import org.apache.qpid.proton.amqp.messaging.AmqpSequence;
 import org.apache.qpid.proton.amqp.messaging.AmqpValue;
 import org.apache.qpid.proton.amqp.messaging.Data;
-import org.apache.qpid.proton.amqp.messaging.MessageAnnotations;
-import org.apache.qpid.proton.codec.impl.DataImpl;
 import org.apache.qpid.proton.message.Message;
 import org.junit.Test;
 
@@ -53,11 +52,10 @@ public class AmqpJmsTextMessageFacadeTest extends AmqpJmsMessageTypesTestCase {
     public void testNewMessageToSendContainsMessageTypeAnnotation() throws Exception {
         AmqpJmsTextMessageFacade amqpTextMessageFacade = createNewTextMessageFacade();
 
-        Message protonMessage = amqpTextMessageFacade.getAmqpMessage();
-        MessageAnnotations annotations = protonMessage.getMessageAnnotations();
-        Map<Symbol, Object> annotationsMap = annotations.getValue();
+        assertNotNull("MessageAnnotations section was not present", amqpTextMessageFacade.getMessageAnnotations());
+
+        Map<Symbol, Object> annotationsMap = amqpTextMessageFacade.getMessageAnnotations().getValue();
 
-        assertNotNull("MessageAnnotations section was not present", annotations);
         assertNotNull("MessageAnnotations section value was not present", annotationsMap);
 
         assertTrue("expected message type annotation to be present", annotationsMap.containsKey(AmqpMessageSupport.getSymbol(JMS_MSG_TYPE)));
@@ -105,9 +103,9 @@ public class AmqpJmsTextMessageFacadeTest extends AmqpJmsMessageTypesTestCase {
         AmqpJmsTextMessageFacade amqpTextMessageFacade = createNewTextMessageFacade();
 
         amqpTextMessageFacade.setText(text);
-        assertNotNull(amqpTextMessageFacade.getAmqpMessage().getBody());
-        assertTrue(amqpTextMessageFacade.getAmqpMessage().getBody() instanceof AmqpValue);
-        assertEquals(text, ((AmqpValue) amqpTextMessageFacade.getAmqpMessage().getBody()).getValue());
+        assertNotNull(amqpTextMessageFacade.getBody());
+        assertTrue(amqpTextMessageFacade.getBody() instanceof AmqpValue);
+        assertEquals(text, ((AmqpValue) amqpTextMessageFacade.getBody()).getValue());
 
         assertEquals(text, amqpTextMessageFacade.getText());
     }
@@ -174,7 +172,7 @@ public class AmqpJmsTextMessageFacadeTest extends AmqpJmsMessageTypesTestCase {
 
     @Test
     public void testGetTextUsingReceivedMessageWithZeroLengthDataSectionReturnsEmptyString() throws Exception {
-        org.apache.qpid.proton.codec.Data payloadData = new DataImpl();
+        org.apache.qpid.proton.codec.Data payloadData = org.apache.qpid.proton.codec.Data.Factory.create();
         payloadData.putDescribedType(new DataDescribedType(new Binary(new byte[0])));
         Binary b = payloadData.encode();
 
@@ -191,7 +189,7 @@ public class AmqpJmsTextMessageFacadeTest extends AmqpJmsMessageTypesTestCase {
         String encodedString = "myEncodedString";
         byte[] encodedBytes = encodedString.getBytes(Charset.forName("UTF-8"));
 
-        org.apache.qpid.proton.codec.Data payloadData = new DataImpl();
+        org.apache.qpid.proton.codec.Data payloadData = org.apache.qpid.proton.codec.Data.Factory.create();
         payloadData.putDescribedType(new DataDescribedType(new Binary(encodedBytes)));
         Binary b = payloadData.encode();
 
@@ -203,6 +201,14 @@ public class AmqpJmsTextMessageFacadeTest extends AmqpJmsMessageTypesTestCase {
         assertEquals(encodedString, amqpTextMessageFacade.getText());
     }
 
+    @Test
+    public void testGetTextWithNonAmqpValueOrDataSectionReportsNoBody() throws Exception {
+        Message message = Message.Factory.create();
+        message.setBody(new AmqpSequence(new ArrayList<Object>()));
+        AmqpJmsTextMessageFacade amqpTextMessageFacade = createReceivedTextMessageFacade(createMockAmqpConsumer(), message);
+        assertFalse(amqpTextMessageFacade.hasBody());
+    }
+
     @Test
     public void testGetTextWithNonAmqpValueOrDataSectionThrowsISE() throws Exception {
         Message message = Message.Factory.create();
