@@ -47,6 +47,7 @@ import org.apache.qpid.jms.meta.JmsProducerInfo;
 import org.apache.qpid.jms.meta.JmsSessionInfo;
 import org.apache.qpid.jms.provider.DefaultProviderListener;
 import org.apache.qpid.jms.provider.ProviderFuture;
+import org.apache.qpid.jms.provider.ProviderFutureFactory;
 import org.apache.qpid.jms.test.Wait;
 import org.junit.After;
 import org.junit.Before;
@@ -61,6 +62,8 @@ public class FailoverProviderTest extends FailoverProviderTestSupport {
 
     private static final Logger LOG = LoggerFactory.getLogger(FailoverProviderTest.class);
 
+    private final ProviderFutureFactory futuresFactory = ProviderFutureFactory.create(Collections.emptyMap());
+
     private List<URI> uris;
     private FailoverProvider provider;
     private JmsConnectionInfo connection;
@@ -91,7 +94,7 @@ public class FailoverProviderTest extends FailoverProviderTestSupport {
 
     @Test(timeout = 30000)
     public void testCreateProviderOnlyUris() {
-        provider = new FailoverProvider(uris);
+        provider = new FailoverProvider(uris, futuresFactory);
         assertEquals(FailoverUriPool.DEFAULT_RANDOMIZE_ENABLED, provider.isRandomize());
         assertNull(provider.getRemoteURI());
         assertNotNull(provider.getNestedOptions());
@@ -103,7 +106,7 @@ public class FailoverProviderTest extends FailoverProviderTestSupport {
         Map<String, String> options = new HashMap<String, String>();
         options.put("transport.tcpNoDelay", "true");
 
-        provider = new FailoverProvider(options);
+        provider = new FailoverProvider(options, futuresFactory);
         assertEquals(FailoverUriPool.DEFAULT_RANDOMIZE_ENABLED, provider.isRandomize());
         assertNull(provider.getRemoteURI());
         assertNotNull(provider.getNestedOptions());
@@ -113,7 +116,7 @@ public class FailoverProviderTest extends FailoverProviderTestSupport {
 
     @Test(timeout = 30000)
     public void testCreateProviderWithNestedOptions() {
-        provider = new FailoverProvider(uris, Collections.<String, String>emptyMap());
+        provider = new FailoverProvider(uris, Collections.<String, String>emptyMap(), futuresFactory);
         assertEquals(FailoverUriPool.DEFAULT_RANDOMIZE_ENABLED, provider.isRandomize());
         assertNull(provider.getRemoteURI());
         assertNotNull(provider.getNestedOptions());
@@ -122,7 +125,7 @@ public class FailoverProviderTest extends FailoverProviderTestSupport {
 
     @Test(timeout = 30000)
     public void testProviderListener() {
-        provider = new FailoverProvider(uris, Collections.<String, String>emptyMap());
+        provider = new FailoverProvider(uris, Collections.<String, String>emptyMap(), futuresFactory);
         assertNull(provider.getProviderListener());
         provider.setProviderListener(new DefaultProviderListener());
         assertNotNull(provider.getProviderListener());
@@ -130,14 +133,14 @@ public class FailoverProviderTest extends FailoverProviderTestSupport {
 
     @Test(timeout = 30000)
     public void testGetRemoteURI() throws Exception {
-        provider = new FailoverProvider(uris, Collections.<String, String>emptyMap());
+        provider = new FailoverProvider(uris, Collections.<String, String>emptyMap(), futuresFactory);
 
         assertNull(provider.getRemoteURI());
         provider.connect(connection);
         assertTrue("Should have a remote URI after connect", Wait.waitFor(new Wait.Condition() {
 
             @Override
-            public boolean isSatisified() throws Exception {
+            public boolean isSatisfied() throws Exception {
                 return provider.getRemoteURI() != null;
             }
         }, TimeUnit.SECONDS.toMillis(20), 10));
@@ -145,14 +148,14 @@ public class FailoverProviderTest extends FailoverProviderTestSupport {
 
     @Test(timeout = 30000)
     public void testToString() throws Exception {
-        provider = new FailoverProvider(uris, Collections.<String, String>emptyMap());
+        provider = new FailoverProvider(uris, Collections.<String, String>emptyMap(), futuresFactory);
 
         assertNotNull(provider.toString());
         provider.connect(connection);
         assertTrue("Should have a mock scheme after connect", Wait.waitFor(new Wait.Condition() {
 
             @Override
-            public boolean isSatisified() throws Exception {
+            public boolean isSatisfied() throws Exception {
                 LOG.info("FailoverProvider: toString = {}", provider.toString());
                 return provider.toString().contains("mock://");
             }
@@ -161,7 +164,7 @@ public class FailoverProviderTest extends FailoverProviderTestSupport {
 
     @Test(timeout = 30000)
     public void testConnectToMock() throws Exception {
-        provider = new FailoverProvider(uris, Collections.<String, String>emptyMap());
+        provider = new FailoverProvider(uris, Collections.<String, String>emptyMap(), futuresFactory);
         assertEquals(FailoverUriPool.DEFAULT_RANDOMIZE_ENABLED, provider.isRandomize());
         assertNull(provider.getRemoteURI());
 
@@ -177,7 +180,7 @@ public class FailoverProviderTest extends FailoverProviderTestSupport {
 
         provider.connect(connection);
 
-        ProviderFuture request = new ProviderFuture();
+        ProviderFuture request = provider.newProviderFuture();
         provider.create(createConnectionInfo(), request);
 
         request.sync(10, TimeUnit.SECONDS);
@@ -192,7 +195,7 @@ public class FailoverProviderTest extends FailoverProviderTestSupport {
 
     @Test(timeout = 30000)
     public void testCannotStartWithoutListener() throws Exception {
-        provider = new FailoverProvider(uris, Collections.<String, String>emptyMap());
+        provider = new FailoverProvider(uris, Collections.<String, String>emptyMap(), futuresFactory);
         assertEquals(FailoverUriPool.DEFAULT_RANDOMIZE_ENABLED, provider.isRandomize());
         assertNull(provider.getRemoteURI());
         provider.connect(connection);
@@ -231,9 +234,9 @@ public class FailoverProviderTest extends FailoverProviderTestSupport {
     }
 
     @Test(timeout = 30000)
-    public void testMaxReconnectAttempts() throws Exception {
+    public void testMaxReconnectAttemptsWithOneURI() throws Exception {
         JmsConnectionFactory factory = new JmsConnectionFactory(
-            "failover:(mock://localhost?mock.failOnConnect=true)" +
+            "failover:(mock://localhost1?mock.failOnConnect=true)" +
             "?failover.maxReconnectAttempts=5" +
             "&failover.useReconnectBackOff=false");
 
@@ -254,6 +257,34 @@ public class FailoverProviderTest extends FailoverProviderTestSupport {
         assertEquals(5, mockPeer.getContextStats().getCloseAttempts());
     }
 
+    @Test(timeout = 30000)
+    public void testMaxReconnectAttemptsWithMultipleURIs() throws Exception {
+        JmsConnectionFactory factory = new JmsConnectionFactory(
+            "failover:(mock://192.168.2.1?mock.failOnConnect=true," +
+                      "mock://192.168.2.2?mock.failOnConnect=true," +
+                      "mock://192.168.2.3?mock.failOnConnect=true)" +
+            "?failover.maxReconnectAttempts=5" +
+            "&failover.reconnectDelay=1" +
+            "&failover.useReconnectBackOff=false");
+
+        Connection connection = null;
+        try {
+            connection = factory.createConnection();
+            connection.start();
+            fail("Should have stopped after five retries.");
+        } catch (JMSException ex) {
+        } finally {
+            if (connection != null) {
+                connection.close();
+            }
+        }
+
+        // The number should scale by the number of URIs in the list
+        assertEquals(15, mockPeer.getContextStats().getProvidersCreated());
+        assertEquals(15, mockPeer.getContextStats().getConnectionAttempts());
+        assertEquals(15, mockPeer.getContextStats().getCloseAttempts());
+    }
+
     @Test(timeout = 30000)
     public void testMaxReconnectAttemptsWithBackOff() throws Exception {
         JmsConnectionFactory factory = new JmsConnectionFactory(
@@ -393,7 +424,7 @@ public class FailoverProviderTest extends FailoverProviderTestSupport {
         final long SEND_TIMEOUT = TimeUnit.SECONDS.toMillis(6);
         final long REQUEST_TIMEOUT = TimeUnit.SECONDS.toMillis(7);
 
-        provider = new FailoverProvider(uris, Collections.<String, String>emptyMap());
+        provider = new FailoverProvider(uris, Collections.<String, String>emptyMap(), futuresFactory);
         provider.setProviderListener(new DefaultProviderListener() {
 
             @Override
@@ -411,7 +442,7 @@ public class FailoverProviderTest extends FailoverProviderTestSupport {
         connectionInfo.setSendTimeout(SEND_TIMEOUT);
         connectionInfo.setRequestTimeout(REQUEST_TIMEOUT);
 
-        ProviderFuture request = new ProviderFuture();
+        ProviderFuture request = provider.newProviderFuture();
         provider.create(connectionInfo, request);
         request.sync();
 
@@ -422,13 +453,13 @@ public class FailoverProviderTest extends FailoverProviderTestSupport {
 
     @Test(timeout = 30000)
     public void testAmqpOpenServerListActionDefault() {
-        provider = new FailoverProvider(uris);
+        provider = new FailoverProvider(uris, futuresFactory);
         assertEquals("REPLACE", provider.getAmqpOpenServerListAction());
     }
 
     @Test(timeout = 30000)
     public void testSetGetAmqpOpenServerListAction() {
-        provider = new FailoverProvider(uris);
+        provider = new FailoverProvider(uris, futuresFactory);
         String action = "ADD";
         assertFalse(action.equals(provider.getAmqpOpenServerListAction()));
 
@@ -438,7 +469,7 @@ public class FailoverProviderTest extends FailoverProviderTestSupport {
 
     @Test(timeout = 30000)
     public void testSetInvalidAmqpOpenServerListActionThrowsIAE() {
-        provider = new FailoverProvider(uris);
+        provider = new FailoverProvider(uris, futuresFactory);
         try {
             provider.setAmqpOpenServerListAction("invalid");
             fail("no exception was thrown");
